/*

  This is a part of the LiteStep Shell Source code.

  Copyright (C) 1997-98 Francis Gastellu
                    aka Lone Runner/Aegis

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

/****************************Updates & Additions*****************************

 ****************************************************************************/


#include <windows.h>
#include <stdio.h>

#include "jdesk.h"
#include "../../lssrc/ls-b24/lsapi/lsapi.h"

// -------------------------------------------------------------------------------------------------------

const char rcsRevision[] = "$Revision: 0.30 $"; // Our Version 
const char rcsId[] = "$Id: jdesk.c,v 0.30 1999/08/09 00:22:00 jugg $"; // The Full RCS ID.
const char szAppName[] = "DesktopBackgroundClass"; // Our window class, etc

HWND hMainWnd; // main window handle
HWND hContWnd; // LS window handle
HWND parent; // Which is actually also LS window handle

char lsPath[256];

HINSTANCE dll;
BOOL setDesktopArea = FALSE;
int sdaLeft=0, sdaRight=0, sdaBottom=0, sdaTop=0;
int screenWidth=1024, screenHeight=768;
char mButton1[256], mButton2[256], mButton3[256];
// -------------------------------------------------------------------------------------------------------
// our window procedure
LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam);
void setMinMax(void);
void resetMinMax(void);
// -------------------------------------------------------------------------------------------------------


int SetupDesktop(void)
{
	setDesktopArea = GetRCBool("SetDesktopArea", TRUE);
	sdaLeft = GetRCInt("SDALeft", 0);
	sdaRight = GetRCInt("SDARight", 640);
	sdaTop = GetRCInt("SDATop", 0);
	sdaBottom = GetRCInt("SDABottom", 480);

	GetRCString("DeskMButton1", mButton1, "!none", 256);
	GetRCString("DeskMButton2", mButton2, "!Popup", 256);
	GetRCString("DeskMButton3", mButton3, "!none", 256);

  return 0;
}

// -------------------------------------------------------------------------------------------------------
// Initialization of the module
// -------------------------------------------------------------------------------------------------------
int initModuleEx(HWND ParentWnd, HINSTANCE dllInst, LPCSTR szPath)
{
	WNDCLASS wc;
	UINT Msgs[3];

	dll = dllInst;
	parent = ParentWnd;
	strcpy (lsPath, szPath);

	SetupDesktop();

	screenWidth = GetSystemMetrics(SM_CXSCREEN);
	screenHeight = GetSystemMetrics(SM_CYSCREEN);

	// Register Desktop window class
	memset(&wc,0,sizeof(wc));
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);
	wc.lpfnWndProc = WndProc;				// our window procedure
	wc.hInstance = dllInst;					// hInstance of DLL
	wc.lpszClassName = szAppName;		// our window class name
	wc.style = 0;
	if (!RegisterClass(&wc))
	{
		MessageBox(parent,"Error registering window class","jDesk",MB_OK);
		return 1;
	}
	// Find the litestep main window
	hContWnd = GetLitestepWnd();

	// Set the maximized size for applications
	setMinMax();

	// Create jDesk window
	hMainWnd = CreateWindowEx(
		WS_EX_TOOLWINDOW,					// exstyles 
		szAppName,								// our window class name
		"",												// use description for a window title
		WS_POPUP|WS_CLIPSIBLINGS|WS_CLIPCHILDREN,                    
		0, 0,											// position 
		screenWidth,screenHeight,	// width & height of window
		hContWnd,									// parent window 
		NULL,											// no menu
		dll,											// hInstance of DLL
		NULL);										// no window creation data
	if (!hMainWnd)
	{
		MessageBox(parent,"Error creating window","jDesk",MB_OK);
		return 1;
	}

	Msgs[0] = LM_GETREVID;
	Msgs[1] = LM_REPAINT;
	Msgs[2] = 0;
	SendMessage(parent, LM_REGISTERMESSAGE, (WPARAM) hMainWnd, (LPARAM) Msgs);

	SetWindowLong(hMainWnd, GWL_USERDATA, magicDWord);
   
	// show the window
	//SetCursor(LoadCursor(NULL,IDC_ARROW));
	SetWindowPos(hMainWnd, HWND_BOTTOM, 0, 0, 0, 0, SWP_NOSIZE|SWP_NOMOVE);
	ShowWindow(hMainWnd,SW_SHOWNORMAL);

	SetActiveWindow(hMainWnd);
	return 0;
}

// -------------------------------------------------------------------------------------------------------
// cleanup (opposite of init()). Destroys the window, unregisters the window class
void quitModule(HINSTANCE dllInst)
{ 
	int Msgs[3];
    
	resetMinMax();
	DestroyWindow(hMainWnd); // delete our window

	Msgs[0] = LM_GETREVID;
	Msgs[1] = LM_REPAINT;
	Msgs[2] = 0;
	SendMessage(parent, LM_UNREGISTERMESSAGE, (WPARAM) hMainWnd, (LPARAM) Msgs);

	UnregisterClass(szAppName,dllInst/*this_mod->hDllInstance*/); // unregister window class
	return;
}

// -------------------------------------------------------------------------------------------------------
// window procedure for our window
LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
		case LM_GETREVID:
			{
				char *buf = (char *) lParam;

				if (wParam == 0)
				{
					strcpy(buf, "jdesk.dll: ");
					strcat(buf, &rcsRevision[11]);
					buf[strlen(buf)-1] = '\0';
				}
				else if (wParam == 1)
				{
					strcpy(buf, &rcsId[1]);
					buf[strlen(buf)-1] = '\0';
				}
				else
					strcpy(buf, "");
				return strlen(buf);
			}
		case WM_DISPLAYCHANGE:
			{// Screen resolution has changed. Change things accordingly
				screenWidth = LOWORD(lParam); 
				screenHeight = HIWORD(lParam);
				return 0;
			}
		case WM_MOUSEACTIVATE:
			{
				PostMessage(parent, LM_LSSELECT, 0, 0); // Tells litestep it has been selected
				SendMessage(parent, LM_HIDEPOPUP, (int)HIWORD(lParam), (int)LOWORD(lParam)); // Close any popup open
				return MA_ACTIVATE;
			}
		case WM_ACTIVATE:
			{
				SendMessage(parent, LM_LSSELECT, 0, 0); // Tells litestep it has been selected
				if (LOWORD(wParam)) SetActiveWindow(parent);
				return 0;
			}
		case WM_ENDSESSION:
		case WM_QUERYENDSESSION:
			return SendMessage(parent,message,wParam,lParam);
//		case WM_CREATE: return 0;
//		case WM_ERASEBKGND: return 0;
//		case LM_REPAINT: // Manually ask for a paint (stupid)
		case WM_ERASEBKGND:
			{
				PaintDesktop((HDC)wParam);
				return 1;
			}
/*		case WM_PAINT:
			{ // update from doublebuffer
				PAINTSTRUCT ps;
				HDC hdc = BeginPaint(hwnd,&ps);
//				HDC src = CreateCompatibleDC(NULL);

				PaintDesktop(hdc);
//				if (np++ == 0) // If this is the first time we paint, tell it to litestep // And the point of this is !?! I don't even think its possible -jugg
//					PostMessage(parent, 9184, 0, 0);
				EndPaint(hwnd,&ps);
//				DeleteDC(src);
			}
			return 0;*/
		case WM_SYSCOMMAND:
			{
				switch (wParam)
				{
					case SC_CLOSE: // stupid but works
						PostMessage(parent,WM_KEYDOWN,8889,0);
						return 0;
					default:
						break;
				}
				break;
			}
		case WM_KEYDOWN:
		case WM_KEYUP: // forward
				PostMessage(parent,message,wParam,lParam);
			return 0;
		case WM_HOTKEY:
		case WM_WINDOWPOSCHANGING:
			{
				// Trap windowposchanging messages
				WINDOWPOS *c = (WINDOWPOS*)lParam;
				c->hwndInsertAfter = HWND_BOTTOM;
				// c->flags |= SWP_NOZORDER;
				c->flags |= SWP_NOMOVE|SWP_NOSIZE;
				return 0;
			}
/*		case WM_NCPAINT: // one more security
			{
				if (!wParam)
					SetWindowPos(hwnd, HWND_BOTTOM, 0, 0, 0, 0, SWP_NOSIZE|SWP_NOMOVE);
				return 0;
			}*/
		case WM_RBUTTONUP: // Open popup menu
			{
				//SendMessage(parent, LM_POPUP, (int)HIWORD(lParam), (int)LOWORD(lParam));
				ParseBangCommand(hMainWnd, mButton2, NULL);
				return 0;
			}
		case WM_LBUTTONUP:
			{
				ParseBangCommand(hMainWnd, mButton1, NULL);
				//SendMessage(parent, LM_HIDEPOPUP, (int)HIWORD(lParam), (int)LOWORD(lParam));
				return 0;
			}
		case WM_MBUTTONUP:
			{
				ParseBangCommand(hMainWnd, mButton3, NULL);
				//SendMessage(parent, LM_HIDEPOPUP, (int)HIWORD(lParam), (int)LOWORD(lParam));
				return 0;
			}
		case WM_RBUTTONDOWN: // Close popup menu
		case WM_LBUTTONDOWN:
		case WM_MBUTTONDOWN:
				SendMessage(parent, LM_HIDEPOPUP, (int)HIWORD(lParam), (int)LOWORD(lParam));
			return 0;
	}
	return DefWindowProc(hwnd,message,wParam,lParam);
}


// -------------------------------------------------------------------------------------------------------
// Sets maximized windows size
// -------------------------------------------------------------------------------------------------------
void setMinMax(void)
{
	RECT r;
	r.top = 0;
	r.left = 0;
	r.bottom = screenHeight;
	r.right = screenWidth;
	
	if (setDesktopArea)
	{/*Allowing a minimum of 100x100 desktop area. Anything less is even more rediculous then that, plus impossible to access a max'd window.*/
		r.left = sdaLeft<0 ? screenWidth+sdaLeft:sdaLeft;
			if (r.left >= screenWidth-100) r.left = 0;
		r.top = sdaTop<0 ? screenHeight+sdaTop:sdaTop;
			if (r.top >= screenHeight-100) r.top = 0;
		r.right = sdaRight<0 ? screenWidth+sdaRight:sdaRight;
			if (r.right <= r.left+100) r.right = screenWidth;
		r.bottom = sdaBottom<0 ? screenHeight+sdaBottom:sdaBottom;
			if (r.bottom <= r.top+100) r.bottom = screenHeight;
	}
	SystemParametersInfo(SPI_SETWORKAREA,0,&r,SPIF_SENDCHANGE);
	return;
}

// -------------------------------------------------------------------------------------------------------
// Reset to full screen
// -------------------------------------------------------------------------------------------------------
void resetMinMax(void)
{
	RECT r;	
	r.top = 0;
	r.left = 0;
	r.bottom = screenHeight;
	r.right = screenWidth;
	SystemParametersInfo(SPI_SETWORKAREA,0,&r,SPIF_SENDCHANGE);
	return;
}
